import itertools
import pandas as pd
from pandas import Series

from .queries import *
from .client import Client

__author__ = "Aisha Mohamed <ahmohamed@qf.org.qa>"


class RDFGraphDataset(object):
	"""
	A class for loading and processing RDF datasets for relational learning models
	It provides some convenience functions for accessing a knowldge graph from a sparql endpoint
	"""

	def __init__(self, sparql_endpoint, graph_name=None):
		"""
		Initialize the RDFGraphDataset class.
		Each graph has to create a new RDFGraphDataset dataset.
		:param sparql_endpoint: string representing the sparql endpoint URI
		:param graph_name: string representing the dataset URI. if no graph_name is passed,
		the sparql queries will not contain FROM clause
		"""
		super(RDFGraphDataset, self).__init__()
		self.graph = graph_name
		self.endpoint = sparql_endpoint
		self.client = Client(self.endpoint)
		self.entity2idx = None
		self.predicate2idx = None
		self.relation2idx = None
		self.attribute2idx = None
		self.attribute_literal_pair2idx = None

	def num_entities(self):
		"""
		:return: integer representing the number of entities
		"""
		query_string = str(NEntities(self.graph))
		result = self.client.execute_query(query_string, limit=1)
		result = result.values.tolist()[0][0]
		return result

	def num_predicates(self):
		"""
		:return: integer representing the number of relations
		"""
		query_string = str(NPredicates(self.graph))
		result = self.client.execute_query(query_string, limit=1)
		result = result.values.tolist()[0][0]
		return result

	def num_relations(self):
		"""
		:return: integer representing the number of relations
		"""
		query_string = str(NRelations(self.graph))
		result = self.client.execute_query(query_string, limit=1)
		result = result.values.tolist()[0][0]
		return result

	def num_attributes(self):
		"""
		:return: integer representing the number of attributes
		"""
		query_string = str(NAttributes(self.graph))
		result = self.client.execute_query(query_string, limit=1)
		result = result.values.tolist()[0][0]
		return result

	def num_attr_literal_pairs(self):
		"""
		:return: integer representing the number of attribute literal pairs
		"""
		query_string = str(NAttributeLiteralPairs(self.graph))
		result = self.client.execute_query(query_string, limit=1)
		result = result.values.tolist()[0][0]
		return result

	def num_triples(self):
		"""
		:return: integer representing the number of triples
		"""
		query_string = str(NTriples(self.graph))
		result = self.client.execute_query(query_string, limit=1)
		result = result.values.tolist()[0][0]
		return result

	def num_entity2entity_triples(self):
		"""
		:return: integer representing the number of entity to entity triples
		"""
		query_string = str(NE2ETriples(self.graph))
		result = self.client.execute_query(query_string, limit=1)
		result = result.values.tolist()[0][0]
		return result

	def num_entity2literal_triples(self):
		"""
		:return: integer representing the number of entity to literal triples
		"""
		query_string = str(NE2LTriples(self.graph))
		result = self.client.execute_query(query_string, limit=1)
		result = result.values.tolist()[0][0]
		return result

	def num_rdf_type_triples(self):
		query_string = str(NRDFTTypeTriples(self.graph))
		result = self.client.execute_query(query_string, limit=1)
		result = result.values.tolist()[0][0]
		return result

	def entities(self, return_format='dict'):
		"""
		A function that returns the entities in the graph
		:param return_format: one of ['dict', 'df', 'list']
		:return: the entities in the knowledge graph represented in the specified return format
		"""
		query_string = str(Entities(self.graph))
		result_df = self.client.execute_query(query_string)
		# set the column name to "entity"
		result_df.columns = ['entity']
		# create a new column for the index
		result_df.reset_index(level=0, inplace=True)
		entity2idx = Series(result_df['index'].values, index=result_df['entity'].values).to_dict()
		self.entity2idx = entity2idx

		if return_format == 'dict':
			return entity2idx
		elif return_format == 'df':
			return result_df
		elif return_format == 'list':
			return list(itertools.chain(*result_df.values))

	def predicates(self, return_format='dict'):
		"""
		A function that returns the predicates in the graph
		:param return_format: one of ['dict', 'df', 'list']
		:return: the predicates in the knowledge graph represented in the specified return format
		"""
		query_string = str(Predicates(self.graph))
		result_df = self.client.execute_query(query_string)
		# set the column name to "predicate"
		result_df.columns = ['predicate']
		# create a new column for the index
		result_df.reset_index(level=0, inplace=True)
		# convert it to a dictionary and store it
		predicate2idx = Series(result_df['index'].values, index=result_df['predicate'].values).to_dict()
		self.predicate2idx = predicate2idx

		if return_format == 'dict':
			return predicate2idx
		elif return_format == 'df':
			return result_df
		elif return_format == 'list':
			return result_df['predicate'].values	

	def relations(self, return_format='dict'):
		"""
		A function that returns the relations in the graph
		:param return_format: one of ['dict', 'df', 'list']
		:return: the relations in the knowledge graph represented in the specified return format
		"""
		query_string = str(Relations(self.graph))
		result_df = self.client.execute_query(query_string)
		# set the column name to "relation"
		result_df.columns = ['relation']
		# create a new column for the index
		result_df.reset_index(level=0, inplace=True)
		relation2idx = Series(result_df['index'].values, index=result_df['relation'].values).to_dict()
		self.relation2idx = relation2idx

		if return_format == 'dict':
			return relation2idx
		elif return_format == 'df':
			return result_df
		elif return_format == 'list':
			return list(itertools.chain(*result_df.values))

	def attributes(self, return_format='dict'):
		"""
		A function that returns the attributes in the graph
		:param return_format: one of ['dict', 'df', 'list']
		:return: the attributes in the knowledge graph represented in the specified return format		
		"""
		query_string = str(Attributes(self.graph))
		result_df = self.client.execute_query(query_string)
		# set the column name to "attribute"
		result_df.columns = ['attribute']
		# create a new column for the index
		result_df.reset_index(level=0, inplace=True)
		attribute2idx = Series(result_df['index'].values, index=result_df['attribute'].values).to_dict()
		self.attribute2idx = attribute2idx

		if return_format == 'dict':
			return attribute2idx
		elif return_format == 'df':
			return result_df
		elif return_format == 'list':
			return list(itertools.chain(*result_df.values))

	def attr_literal_pairs(self, return_format='dict'):
		"""
		A function that returns the attribute literal pairs in the graph
		:param return_format: one of ['dict', 'df', 'list']
		:return: the attribute literal pairs in the knowledge graph represented in the specified return format
		"""
		query_string = str(AttributeLiteralPairs(self.graph))
		result_df = self.client.execute_query(query_string)
		# set the column name to "attribute_literal_pair"
		result_df.columns = ['attribute', 'literal']
		result_df["attribute_literal_pair"] = result_df["attribute"] + result_df["literal"].map(str)

		#result_df['attribute_literal_pair'] = result_df[['attribute', 'literal']].apply(lambda x: ','.join(str(x)), axis=1)
		print("IN attr_literal_pairs", result_df.columns)
		print("IN attr_literal_pairs", result_df.head())
		# create a new column for the index
		result_df.reset_index(level=0, inplace=True)
		attribute_literal_pair2idx = Series(result_df['index'].values, index=result_df['attribute_literal_pair'].values).to_dict()
		self.attribute_literal_pair2idx = attribute_literal_pair2idx

		if return_format == 'dict':
			return attribute_literal_pair2idx
		elif return_format == 'df':
			return result_df
		elif return_format == 'list':
			return list(itertools.chain(*result_df.values))

	def triples(self, entity2idx=None, predicate2idx=None, return_format='list'):
		"""
		A function that returns all the triples in the specified graph as indices rather than URIs
		:param entity2idx: a dictionary mapping each entity in the graph to an index from 0 to n_entities-1
		:param predicate2idx: a dictionary mapping each entity in the graph to an index from 0 to n_predicates-1
		:param return_format: one of ['list', 'df']
		:return: the triples in the graph in the specified format
		"""
		query_string = str(Triples(self.graph))
		result_df = self.client.execute_query(query_string)
		result_df.columns = ['subject', 'object', 'predicate']
		# find the dictionary mapping each entity to its index
		if entity2idx is None:
			if self.entity2idx is None:
				entity2idx = self.entities('dict')
			else:
				entity2idx = self.entity2idx
		# find the dictionary mapping each predicate to its index
		if predicate2idx is None:
			if self.predicate2idx is None:
				predicate2idx = self.predicates('dict')
			else:
				predicate2idx = self.predicate2idx
		# map the entities and predicates to their indices
		result_df.replace({'subject': entity2idx}, inplace=True)
		result_df.replace({'object': entity2idx}, inplace=True)
		result_df.replace({'predicate': predicate2idx}, inplace=True)

		if return_format == 'list':
			return result_df.values.tolist()
		elif return_format == 'df':
			return result_df

	def entity2entity_triples(self, entity2idx=None, relation2idx=None, return_format='list'):
		"""
		A function that returns all the entity2entity triples in the specified graph as indices rather than URIs
		:param entity2idx: a dictionary mapping each entity in the graph to an index from 0 to n_entities-1
		:param relation2idx: a dictionary mapping each entity in the graph to an index from 0 to n_relations-1
		:param return_format: one of ['list', 'df']
		:return: the triples in the graph in the specified format
		"""
		query_string = str(E2ETriples(self.graph))
		result_df = self.client.execute_query(query_string)
		rdf_triples_df = self.client.execute_query(str(RDFTTypeTriples(self.graph)))
		# find the dictionary mapping each entitt to its index
		if entity2idx is None:
			if self.entity2idx is None:
				entity2idx = self.entities('dict')
			else:
				entity2idx = self.entity2idx
		# find the dictionary mapping each relation to its index
		if relation2idx is None:
			if self.relation2idx is None:
				relation2idx = self.relations('dict')
			else:
				relation2idx = self.relation2idx

		# rearrange columns to be subject, object, relation
		result_df.columns = ['subject', 'predicate', 'object']
		rdf_triples_df.columns = ['subject', 'predicate', 'object']
		result_df = pd.concat([result_df, rdf_triples_df], ignore_index=True)
		#result_df = result_df[['subject', 'object', 'predicate']]
		# map the subjects to their indices
		result_df = result_df.replace({'subject': entity2idx})
		result_df = result_df.replace({'object': entity2idx})
		result_df = result_df.replace({'predicate': relation2idx})

		if return_format == 'list':
			return result_df.values.tolist()
		elif return_format == 'df':
			return result_df

	def entity2literal_triples(self, entity2idx=None, attribute_literal_pair2idx=None, return_format='list'):
		"""
		A function that returns all the entity2literal triples in the specified graph as indices rather than URIs
		:param entity2idx: a dictionary mapping each entity in the graph to an index from 0 to n_entities-1
		:param attribute_literal_pair2idx: a dictionary mapping each entity in the graph to an index from 0 to
		n_relations-1
		:param return_format: one of ['list', 'df']
		:return: the triples in the graph in the specified format
		"""
		query_string = str(E2LTriples(self.graph))
		result_df = self.client.execute_query(query_string)
		# find the dictionary mapping each entity to its index
		if entity2idx is None:
			if self.entity2idx is None:
				entity2idx = self.entities('dict')
			else:
				entity2idx = self.entity2idx
		# find the dictionary mapping each predicate,literal pair to its index
		if attribute_literal_pair2idx is None:
			if self.attribute_literal_pair2idx is None:
				attribute_literal_pair2idx = self.attr_literal_pairs('dict')
			else:
				attribute_literal_pair2idx = self.attribute_literal_pair2idx

		result_df.columns = ['subject', 'predicate', 'object']
		result_df['attribute_literal_pair'] = result_df[['predicate', 'object']].apply(lambda x: ','.join(str(x)), axis=1)

		# map the subjects to their indices
		result_df = result_df.replace({'subject': entity2idx})
		result_df = result_df.replace({'attribute_literal_pair': attribute_literal_pair2idx})

		if return_format == 'list':
			return result_df.values.tolist()
		elif return_format == 'df':
			return result_df

	def subjects(self, p, entity2idx=None, return_format='list'):
		"""
		A function that returns all the subjects of predicate p in the specified graph as indices rather than URIs
		:param entity2idx: a dictionary mapping each entity in the graph to an index from 0 to n_entities-1
		:param return_format: one of ['list', 'df']
		:return: the triples in the graph in the specified format
		"""
		query_string = str(Subjects(self.graph, p))
		result_df = self.client.execute_query(query_string)
		# find the dictionary mapping each entity to its index
		if entity2idx is None:
			if self.entity2idx is None:
				entity2idx = self.entities('dict')
			else:
				entity2idx = self.entity2idx

		result_df.columns = ['subject']
		# map the subjects to their indices
		result_df = result_df.replace({'subject': entity2idx})

		if return_format == 'list':
			return result_df.values.tolist()
		elif return_format == 'df':
			return result_df

	def objects(self, p, entity2idx=None, return_format='list'):
		"""
		A function that returns all the objects of predicate p in the specified graph as indices rather than URIs
		:param entity2idx: a dictionary mapping each entity in the graph to an index from 0 to n_entities-1
		:param return_format: one of ['list', 'df']
		:return: the triples in the graph in the specified format
		"""
		query_string = str(Objects(self.graph, p))
		result_df = self.client.execute_query(query_string)
		# find the dictionary mapping each entity to its index
		if entity2idx is None:
			if self.entity2idx is None:
				entity2idx = self.entities('dict')
			else:
				entity2idx = self.entity2idx

		result_df.columns = ['object']
		# map the subjects to their indices
		result_df = result_df.replace({'object': entity2idx})

		if return_format == 'list':
			return result_df.values.tolist()
		elif return_format == 'df':
			return result_df

	def predicates_freq(self):
		"""
		A function that returns all the predicates and their frequency
		"""
		query_string = str(PredicatesFreq(self.graph))
		result_df = self.client.execute_query(query_string)
		result_df.columns = ['predicate', 'freq']

		if return_format == 'df':
			return result_df







import sys
import io

from SPARQLWrapper import SPARQLWrapper, CSV
import pandas as pd

__author__ = "Aisha Mohamed <ahmohamed@qf.org.qa>"

_MAX_ROWS = 1000000 # maximum number of rows returned in the result set
_TIMEOUT = 900 # in seconds

class Client(object):
    """
    class for sparql client that handles communication with a sparql end-point
    over http using the sparql wrapper library.
    """
    def __init__(self, endpoint):
        """
        Constructs an instance of the client class
        :param endpoint: string of the SPARQL endpoint's URI hostname:port
        :type endpoint: string
        """
        self.endpoint = endpoint

    def is_alive(self, endpoint=None):
        """
        :param endpoint string of the SPARQL endpoint's URI
        :type endpoint string
        :return if endpoint is not None return Ture if endpoint is alive else
            return False. if endpoint is None return True if self.endpoint is
            alive and False otherwise.
        """
        pass

    def get_endpoint(self):
        """
        :return a string of the endpont URI
        """
        return self.endpoint

    def set_endpoint(self, endpoint):
        """
        updates self.endpoint with the new endpoint
        :param endpoint: endpoint uri
        """
        self.endpoint = endpoint

    def execute_query(self, query, limit=_MAX_ROWS):
        """
        Connects to the sparql endpoint, sends the query and returns a dataframe containing the result of a sparql query
        :param query: a valid sparql query string
        :type query: string
        :param output_file: the path to the output file
        :type output_file: string
        :return: a pandas dataframe representing the result of the query
        """
        client = SPARQLWrapper(self.endpoint)
        client.setTimeout(_TIMEOUT)
        offset = 0
        results_string = "" # where all the results are concatenated
        continue_straming = True
        while continue_straming:
            if limit > 1:
                query_string = query+" OFFSET {} LIMIT {}".format(str(offset), str(limit))
            else:
                query_string = query
            client.setQuery(query_string)
            try:
                client.setReturnFormat(CSV)
                header = client.query().convert().split("\n",1)[0]
                results = client.query().convert().split("\n",1)[1] # string
                # if the number of rows is less then the maximum number of rows
                if results.count('\n') < _MAX_ROWS:
                    continue_straming = False
                offset = offset + limit
            except Exception as e:
                print(e)
                sys.exit()
            results_string += results.decode("utf-8")
        # convert it to a dataframe
        results_string = header + "\n" + results_string
        f = io.StringIO(results_string)
        f.seek(0)
        df = pd.read_csv(f, sep=',') # to get the values and the header
        return df


__author__ = "Aisha Mohamed <ahmohamed@qf.org.qa>"


class SparqlQuery(object):
	"""
	A class for the sparql query
	"""
	def __init__(self, graph=None):
		super(SparqlQuery, self).__init__()
		if graph is not None:
			self.graph = ' FROM <'+ graph +'> '
		else:
			self.graph = ' '

	def __str__(self):
		pass


class NTriples(SparqlQuery):
	"""
	A class for the sparql query that returns the number of triples
	"""
	def __str__(self):
		return 'SELECT count(DISTINCT*)' + self.graph + 'WHERE { ?s ?p ?o}'


class NE2ETriples(SparqlQuery):
	"""
	A class for the sparql query that returns the number of triples
	"""
	def __str__(self):
		return 'PREFIX  rdf:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#> SELECT count(*)' + self.graph + \
			'WHERE {{ SELECT DISTINCT ?s ?p ?o WHERE { ?s ?p ?o {SELECT DISTINCT ?p' + self.graph+ \
			'WHERE { ?a ?p ?b . ?b rdf:type ?c}}}} UNION {SELECT DISTINCT ?s (rdf:type as ?p) ?o' + self.graph \
			+ 'WHERE {?s rdf:type ?o }}}'


class NE2LTriples(SparqlQuery):
	"""
	A class for the sparql query that returns the number of triples
	"""
	def __str__(self):
		return 'SELECT count(DISTINCT *)' + self.graph + 'WHERE { {SELECT DISTINCT ?s ?p ?o WHERE {?s ?p ?o . {SELECT DISTINCT ?p'+ self.graph + 'WHERE { ?a ?p ?b . FILTER isLiteral(?b)}}}}}'


class NRDFTTypeTriples(SparqlQuery):
	"""
	A class for the sparql query that returns the number of (?s rdf:type ?c) triples
	"""
	def __str__(self):
		return 'PREFIX  rdf:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#> SELECT count(*) WHERE { {SELECT DISTINCT ?s rdf:type ?o' + self.graph + \
			'WHERE {?s rdf:type ?o }}}'


class NEntities(SparqlQuery):
	"""
	A class for the sparql query that returns the number of entities
	"""
	def __str__(self):
		return 'PREFIX  rdf:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#> SELECT count(DISTINCT ?s)' \
			+ self.graph + 'WHERE {?s rdf:type ?c}'


class NPredicates(SparqlQuery):
	def __str__(self):
		return 'SELECT COUNT(DISTINCT ?p)' + self.graph + 'WHERE { ?s ?p ?o }'


class NRelations(SparqlQuery):
	"""
	A class for the sparql query that returns the number of relations
	"""
	def __str__(self):
		return 'PREFIX  rdf:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#> SELECT count(DISTINCT ?p)' + \
			self.graph + 'WHERE { {?s ?p ?o . ?o rdf:type ?c} UNION {SELECT (rdf:type as ?p) WHERE {}} }'


class NAttributes(SparqlQuery):
	"""
	A class for the sparql query that returns the number of attributes
	"""
	def __str__(self):
		return 'SELECT count(DISTINCT ?p)' + self.graph + 'WHERE { ?s ?p ?o . FILTER isLiteral(?o)}'


class NAttributeLiteralPairs(SparqlQuery):
	"""
	A class for the sparql query that returns the number of attr_literal_pairs
	"""
	def __str__(self):
		return 'SELECT COUNT (*) WHERE {SELECT DISTINCT ?p ?o' + self.graph + \
			'WHERE { ?s ?p ?o . FILTER isLiteral(?o)}}'


class Triples(SparqlQuery):
	"""
	A class for the sparql query that returns the triples
	"""
	def __str__(self):
		return 'SELECT DISTINCT ?s ?o ?p' + self.graph + 'WHERE { ?s ?p ?o}'


class E2ETriples(SparqlQuery):
	"""
	A class for the sparql query that returns the number of triples
	"""
	def __str__(self):
		return 'PREFIX  rdf:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#> SELECT DISTINCT ?s ?o ?p' + self.graph + \
			'WHERE {?s ?p ?o . {SELECT ?p WHERE { ?a ?p ?b . ?b rdf:type ?c}}}'


class RDFTTypeTriples(SparqlQuery):
	def __str__(self):
		return 'PREFIX  rdf:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#> SELECT DISTINCT (?s ?o rdf:type as ?p)' + self.graph + \
			'WHERE {?s rdf:type ?o }'


class E2LTriples(SparqlQuery):
	"""
	A class for the sparql query that returns the number of triples
	"""
	def __str__(self):
		return 'SELECT *' + self.graph + 'WHERE { ?s ?p ?o . {SELECT DISTINCT ?p' + \
			'WHERE { ?a ?p ?b . FILTER isLiteral(?b)}}}'


class Entities(SparqlQuery):
	"""
	A class for the sparql query that returns the entities
	"""
	def __str__(self):
		return 'PREFIX  rdf:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#> SELECT DISTINCT ?s' + \
			self.graph + 'WHERE {?s rdf:type ?c}'


class Relations(SparqlQuery):
	"""
	A class for the sparql query that returns the entity to entiity relations
	"""
	def __str__(self):
		return 'PREFIX  rdf:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#> SELECT DISTINCT ?p' + \
			self.graph + 'WHERE { {?s ?p ?o . ?o rdf:type ?c} UNION {SELECT (rdf:type as ?p) WHERE {}} } '


class Attributes(SparqlQuery):
	"""
	A class for the sparql query that returns the  attributes
	"""
	def __str__(self):
		return 'SELECT DISTINCT ?p' + self.graph + 'WHERE { ?s ?p ?o . FILTER isLiteral(?o)}'


class Predicates(SparqlQuery):
	"""
	A class for the sparql query that returns the predicates
	"""
	def __str__(self):
		return 'SELECT DISTINCT ?p' + self.graph + 'WHERE { ?s ?p ?o }'


class AttributeLiteralPairs(SparqlQuery):
	"""
	A class for the sparql query that returns the attr_literal_pairs
	"""
	def __str__(self):
		return 'SELECT DISTINCT ?p ?o' + self.graph + 'WHERE { ?s ?p ?o . FILTER isLiteral(?o)}'


class Subjects(SparqlQuery):
	"""
	A class for the sparql query that returns the subjects of a given predicate p.
	It returns all the subjects ?s that match the following pattern
		?s p ?o
	"""
	def __init__(self, graph, p):
		super(Subjects, self).__init__(graph)
		self.p = p

	def __str__(self):
		return 'SELECT DISTINCT ?s' + self.graph + 'WHERE { ?s <' + self.p + '> ?o }'


class Objects(SparqlQuery):
	"""
	A class for the sparql query that returns the objects of a given predicate p.
	It returns all the objects ?o that match the following pattern
		?s p ?o
	"""
	def __init__(self, graph, p):
		super(Objects, self).__init__(graph)
		self.p = p

	def __str__(self):
		return 'SELECT DISTINCT ?o' + self.graph + 'WHERE { ?s <' + self.p + '> ?o }'


class PredicatesFreq(SparqlQuery):
	"""
	A class for the sparql query that returns the predicates of a given graph.
	It returns all the predicates ?p that match the following pattern
		?s p ?o
	"""
	def __str__(self):
		return 'SELECT ?p COUNT(DISTINCT *)' + self.graph + 'WHERE { ?s ?p ?o } GROUP BY ?p'


class PTriples(SparqlQuery):
	def __init__(self, graph, p):
		super(PTriples, self).__init__(graph)
		self.p = p

	def __str__(self):
		return 'SELECT DISTINCT ?s ?o <'+ self.p + '>' + self.graph + 'WHERE { ?s <' + self.p +'> ?o }'

class NPTriples(SparqlQuery):
	def __init__(self, graph, p):
		super(NPTriples, self).__init__(graph)
		self.p = p

	def __str__(self):
		return 'SELECT COUNT(DISTINCT *)' + self.graph + 'WHERE { ?s <' + self.p +'> ?o }'



import pathlib
from setuptools import setup

# This directory
path = pathlib.Path(__file__).parent

readme_content = (path / "README.md").read_text()

setup(name="rdf2srl",
      version="1.0.2",
      description="Exposes RDF datasets from sparql endpoints for relational learning models in convenient formats",
      long_description=readme_content,
      long_description_content_type="text/markdown",
      url="https://github.com/aishahasmoh/RDF2SRL",
      author="Aisha Mohamed",
      author_email="ahmohamed@qcri.qf.org",
      classifiers=[
            "Programming Language :: Python :: 3.5",
      ],
      packages=["rdf2srl"],
      include_package_data=True,
      install_requires=["SPARQLWrapper", "pandas"],
      entry_points={},
      )



import pandas as pd
import json


from .queries import *
from .client import Client
from .rdfloader import RDFGraphDataset

__author__ = "Aisha Mohamed <ahmohamed@qf.org.qa>"


class SmartRDFGraphDataset(RDFGraphDataset):
	"""
	A class for loading and processing RDF datasets for relational learning models
	It provides some convenience functions for accessing a knowldge graph from a sparql endpoint
	"""
	def __init__(self, sparql_endpoint, graph_name=None):
		super(SmartRDFGraphDataset, self).__init__(sparql_endpoint, graph_name)

	def entity2entity_triples(self, return_format='list', entity2idx=None, relation2idx=None,
		return_dict = False, output_dir=None):
		"""
		A function that returns all the entity2entity triples in the specified graph as indices rather than URIs
		:param entity2idx: a dictionary mapping each entity in the graph to an index from 0 to n_entities-1
		:param relation2idx: a dictionary mapping each entity in the graph to an index from 0 to n_relations-1
		:param return_format: one of ['list', 'df']
		:return: the triples in the graph in the specified format
		"""
		if entity2idx is None:
			if self.entity2idx is None:
				entity2idx = self.entities('dict')
			else:
				entity2idx = self.entity2idx
		# find the dictionary mapping each relation to its index
		if relation2idx is None:
			if self.relation2idx is None:
				relation2idx = self.relations('dict')
			else:
				relation2idx = self.relation2idx

		results_df = pd.DataFrame(columns=['subject', 'object', 'predicate'])
		for relation in relation2idx:
			query_string = str(PTriples(self.graph, relation))
			result_df = self.client.execute_query(query_string)
			result_df.columns = ['subject', 'object', 'predicate']
			result_df['subject'] = result_df['subject'].map(entity2idx)
			result_df['object'] = result_df['object'].map(entity2idx)
			result_df['predicate'] = result_df['predicate'].map(relation2idx)
			results_df = pd.merge(results_df, result_df, how='outer')
		
		if output_dir is not None:
			results_df.to_csv(output_dir+"/entity2entity_triples.csv", index=False)
			with open(output_dir+"/relation2idx.json", 'w') as fp:
				json.dump(relation2idx, fp)
			with open(output_dir+"/entity2idx.json", 'w') as fp:
				json.dump(entity2idx, fp)

		if return_format == 'list':
			if return_dict:
				return results_df.values.tolist(), entity2idx, relation2idx
			return results_df.values.tolist() 
		elif return_format == 'df':
			if return_dict:
				return results_df, entity2idx, relation2idx
			return results_df

	def entity2literal_triples(self, entity2idx=None, attribute_literal_pair2idx=None, return_format='list',
		return_dict = False, output_dir=None):
		"""
		A function that returns all the entity2literal triples in the specified graph as indices rather than URIs
		:param entity2idx: a dictionary mapping each entity in the graph to an index from 0 to n_entities-1
		:param attribute_literal_pair2idx: a dictionary mapping each entity in the graph to an index from 0 to
		n_relations-1
		:param return_format: one of ['list', 'df']
		:return: the triples in the graph in the specified format
		"""
		if entity2idx is None:
			if self.entity2idx is None:
				entity2idx = self.entities('dict')
			else:
				entity2idx = self.entity2idx
		# find the dictionary mapping each relation to its index
		if attribute_literal_pair2idx is None:
			if self.attribute_literal_pair2idx is None:
				attribute_literal_pair2idx = self.attr_literal_pairs('dict')
			else:
				attribute_literal_pair2idx = self.attribute_literal_pair2idx

		attributes = self.attributes('dict')

		results_df = pd.DataFrame(columns=['subject', 'attribute_literal_pair'])
		for attribute in attributes:
			query_string = str(PTriples(self.graph, attribute))
			result_df = self.client.execute_query(query_string)
			result_df.columns = ['subject', 'object', 'predicate']
			result_df["attribute_literal_pair"] = result_df["predicate"] + result_df["object"]
			result_df = result_df.drop(columns=['object', 'predicate'])
			result_df['subject'] = result_df['subject'].map(entity2idx)
			result_df['attribute_literal_pair'] = result_df['attribute_literal_pair'].map(attribute_literal_pair2idx)
			results_df = pd.merge(results_df, result_df, how='outer')
		
		if output_dir is not None:
			results_df.to_csv(output_dir+"/entity2literal_triples.csv", index=False)
			with open(output_dir+"/attribute_literal_pair2idx.json", 'w') as fp:
				json.dump(relation2idx, fp)
			with open(output_dir+"/entity2idx.json", 'w') as fp:
				json.dump(entity2idx, fp)

		if return_format == 'list':
			if return_dict:
				return results_df.values.tolist(), entity2idx, relation2idx
			return results_df.values.tolist() 
		elif return_format == 'df':
			if return_dict:
				return results_df, entity2idx, relation2idx
			return results_df


	def subjects(self, p, entity2idx=None, return_format='list', output_dir=None):
		"""
		A function that returns all the subjects of predicate p in the specified graph as indices rather than URIs
		:param entity2idx: a dictionary mapping each entity in the graph to an index from 0 to n_entities-1
		:param return_format: one of ['list', 'df']
		:return: the triples in the graph in the specified format
		"""
		query_string = str(Subjects(self.graph, p))
		result_df = self.client.execute_query(query_string)
		# find the dictionary mapping each entity to its index
		if entity2idx is None:
			if self.entity2idx is None:
				entity2idx = self.entities('dict')
			else:
				entity2idx = self.entity2idx

		result_df.columns = ['subject']
		# map the subjects to their indices
		result_df['subject'] = result_df['subject'].map(entity2idx)

		if output_dir is not None:
			with open(output_dir+"{}_subjects.csv".format(p), "wb") as f:
    			writer = csv.writer(f)
    			writer.writerows(result_df.values.tolist())

		if return_format == 'list':
			return result_df['subject'].values
		elif return_format == 'df':
			return result_df

	def objects(self, p, entity2idx=None, return_format='list'):
		"""
		A function that returns all the objects of predicate p in the specified graph as indices rather than URIs
		:param entity2idx: a dictionary mapping each entity in the graph to an index from 0 to n_entities-1
		:param return_format: one of ['list', 'df']
		:return: the triples in the graph in the specified format
		"""
		query_string = str(Objects(self.graph, p))
		result_df = self.client.execute_query(query_string)
		# find the dictionary mapping each entity to its index
		if entity2idx is None:
			if self.entity2idx is None:
				entity2idx = self.entities('dict')
			else:
				entity2idx = self.entity2idx

		result_df.columns = ['object']
		# map the subjects to their indices
		result_df['object'] = result_df['object'].map(entity2idx)

		if output_dir is not None:
			with open(output_dir+"{}_objects.csv".format(p), "wb") as f:
    			writer = csv.writer(f)
    			writer.writerows(result_df.values.tolist())
		if return_format == 'list':
			return result_df['object'].values
		elif return_format == 'df':
			return result_df

	def predicates_freq(self):
		predicates = self.predicates('list')
		print("predicates are {}".format(predicates))

		results_df = pd.DataFrame(columns=['predicate', 'frequency'])
		for predicate in predicates:
			query_string = str(NPTriples(self.graph, predicate))
			print(query_string)
			result = self.client.execute_query(query_string).values.tolist()[0][0]
			print(result)
			results_df = results_df.append({'predicate' : predicate , 'frequency' : result} , ignore_index=True)
		return results_df